Package: hmetad
Title: Fit the Meta-D' Model of Confidence Ratings Using 'brms'
Version: 0.1.2
Authors@R: c(
    person("Kevin", "O'Neill", email="kevin.o'neill@ucl.ac.uk", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-7401-9802")),
    person("Stephen", "Fleming", email="stephen.fleming@ucl.ac.uk", role = c("aut", "cph"),
           comment = c(ORCID = "0000-0003-0233-4891"))
  )
Description: Implementation of Bayesian regressions over the meta-d' model of 
    psychological data from two alternative forced choice tasks with 
    ordinal confidence ratings. For more information, see 
    Maniscalco & Lau (2012) <doi:10.1016/j.concog.2011.09.021>.
    The package is a front-end to the 'brms' package, which facilitates a wide
    range of regression designs, as well as tools for efficiently extracting 
    posterior estimates, plotting, and significance testing. 
License: GPL (>= 3)
Depends: R (>= 4.2.0), brms (>= 2.23.0)
Imports: abind (>= 1.4.8), dplyr (>= 1.2.0), glue (>= 1.8.0), posterior
        (>= 1.6.1), rlang (>= 1.2.0), stats, stringr (>= 1.6.0),
        tidybayes (>= 3.0.7), tidyr (>= 1.3.2)
Suggests: colorspace, knitr, purrr (>= 1.2.1), rmarkdown, testthat (>=
        3.0.0), tidyverse (>= 2.0.0)
VignetteBuilder: knitr
Encoding: UTF-8
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
URL: https://metacoglab.github.io/hmetad/,
        https://github.com/metacoglab/hmetad
BugReports: https://github.com/metacoglab/hmetad/issues
NeedsCompilation: no
Packaged: 2026-05-15 15:38:15 UTC; kevin
Author: Kevin O'Neill [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-7401-9802>),
  Stephen Fleming [aut, cph] (ORCID:
    <https://orcid.org/0000-0003-0233-4891>)
Maintainer: Kevin O'Neill <kevin.o'neill@ucl.ac.uk>
Repository: CRAN
Date/Publication: 2026-05-15 20:50:02 UTC
Built: R 4.5.2; ; 2026-05-15 22:12:44 UTC; unix
