# Generated by roxygen2: do not edit by hand

export(add_auroc1_draws)
export(add_auroc1_rvars)
export(add_auroc2_draws)
export(add_auroc2_rvars)
export(add_epred_draws_metad)
export(add_epred_rvars_metad)
export(add_linpred_draws_metad)
export(add_linpred_rvars_metad)
export(add_mean_confidence_draws)
export(add_mean_confidence_rvars)
export(add_metacognitive_bias_draws)
export(add_metacognitive_bias_rvars)
export(add_predicted_draws_metad)
export(add_predicted_rvars_metad)
export(add_roc1_draws)
export(add_roc1_rvars)
export(add_roc2_draws)
export(add_roc2_rvars)
export(add_type1_draws)
export(add_type1_rvars)
export(add_type2_draws)
export(add_type2_rvars)
export(aggregate_metad)
export(auroc1)
export(auroc1_draws)
export(auroc1_rvars)
export(auroc2)
export(auroc2_draws)
export(auroc2_rvars)
export(cor_matrix)
export(cov_matrix)
export(epred_draws_metad)
export(epred_rvars_metad)
export(example_data)
export(example_model)
export(fit_metad)
export(joint_probabilities)
export(joint_response)
export(linpred_draws_metad)
export(linpred_rvars_metad)
export(mean_confidence)
export(mean_confidence_draws)
export(mean_confidence_rvars)
export(metac2_parameters)
export(metacognitive_bias_draws)
export(metacognitive_bias_rvars)
export(metad)
export(metad_pmf)
export(normal_lccdf)
export(normal_lcdf)
export(predicted_draws_metad)
export(predicted_rvars_metad)
export(rmatrixnorm)
export(roc1)
export(roc1_draws)
export(roc1_rvars)
export(roc2)
export(roc2_draws)
export(roc2_rvars)
export(sim_metad)
export(sim_metad_condition)
export(sim_metad_participant)
export(sim_metad_participant_condition)
export(stanvars_metad)
export(to_signed)
export(to_unsigned)
export(type1_draws)
export(type1_probabilities)
export(type1_response)
export(type1_rvars)
export(type2_draws)
export(type2_probabilities)
export(type2_response)
export(type2_rvars)
import(dplyr)
import(rlang)
importFrom(brms,
  bf,
  brm,
  brmsterms,
  custom_family,
  get_dpar,
  rmulti_normal,
  stanvar
)
importFrom(glue,glue)
importFrom(posterior,
  as_rvar,
  rvar_ifelse,
  rvar_mean,
  rvar_sum
)
importFrom(stats,
  dmultinom,
  pnorm,
  rmultinom,
  rnorm
)
