#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import dplyr
#' @import rlang
#' @importFrom brms bf
#' @importFrom brms brm
#' @importFrom brms brmsterms
#' @importFrom brms custom_family
#' @importFrom brms get_dpar
#' @importFrom brms rmulti_normal
#' @importFrom brms stanvar
#' @importFrom glue glue
#' @importFrom posterior rvar_ifelse
#' @importFrom posterior rvar_mean
#' @importFrom posterior rvar_sum
#' @importFrom stats dmultinom
#' @importFrom stats pnorm
#' @importFrom stats rmultinom
#' @importFrom stats rnorm
## usethis namespace: end
NULL
